@@ -280,11 +280,8 @@ ShapeAlgorithms.getIntersectionPoints = function(shape, otherShapes, onShape, on
     // find all intersection points:
     for (i=0; i<otherShapes.length; i++) {
         var otherShape = otherShapes[i];
-        if (isFunction(otherShape.data)) {
-            otherShape = otherShape.data();
-        }
 
-        var sol = shape.getIntersectionPoints(otherShape, onShape, false, true);
+        var sol = shape.getIntersectionPoints(getPtr(otherShape), onShape, false, true);
         for (k=0; k<sol.length; k++) {
             if (!onOtherShapes || otherShape.isOnShape(sol[k])) {
                 intersections.push(sol[k]);
@@ -341,6 +338,10 @@ ShapeAlgorithms.getIntersectingShapes = function(doc, entityId, shape, extend) {
     for (var i=0; i<otherEntityIds.length; i++) {
         var otherEntity = doc.queryEntityDirect(otherEntityIds[i]);
 
+        if (isTextBasedEntity(otherEntity)) {
+            continue;
+        }
+
         // ignore intersection points of same entity
         // self intersection points are handled elsewhere
         var same = otherEntityIds[i]===entityId;
@@ -360,7 +361,7 @@ ShapeAlgorithms.getIntersectingShapes = function(doc, entityId, shape, extend) {
             else {
                 // ignore same shape for block reference entities:
                 for (var k=0; k<s.length; k++) {
-                     if (!shape.equals(s[k].data())) {
+                     if (!shape.equals(getPtr(s[k]))) {
                          ret.push(s[k]);
                      }
                 }
@@ -640,6 +641,12 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
 
     // arcs:
     else if (isArcShape(shape)) {
+        if (cutDist1 > cutDist2) {
+            var dummy = cutDist1;
+            cutDist1 = cutDist2;
+            cutDist2 = dummy;
+        }
+
         rest1 = shape.clone();
         rest2 = shape.clone();
 
@@ -657,7 +664,14 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
         if (!extend) {
             var angleLength1 = rest1.getAngleLength(true);
             var angleLength2 = rest2.getAngleLength(true);
-            if (angleLength1+angleLength2 > shape.getAngleLength()) {
+
+            // rest1 is the same as the segment:
+            var same1 = RMath.fuzzyAngleCompare(rest1.getStartAngle(), segment.getStartAngle()) &&
+                    RMath.fuzzyAngleCompare(rest1.getEndAngle(), segment.getEndAngle()) &&
+                    rest1.isReversed()===segment.isReversed();
+
+            // catch common errors:
+            if (angleLength1+angleLength2 > shape.getAngleLength() || same1) {
                 rest1.trimEndPoint(cutDist2);
                 rest2.trimStartPoint(cutDist1);
 
@@ -798,7 +812,6 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
 
     // polyline:
     else if (isPolylineShape(shape)) {
-        // TODO: make sure point 1 is closer to the start of the polyline:
 
         var closed = shape.isGeometricallyClosed();
         if (closed) {
@@ -821,32 +834,21 @@ ShapeAlgorithms.autoSplitManual = function(shape, cutDist1, cutDist2, cutPos1, c
             rest2.trimStartPoint(cutDist2);
         }
         else {
-//            var l1 = shape.getLengthTo(cutDist1);
-//            if (l1<RS.PointTolerance && cutDist1.isEnd===true) {
-//                l1 = shape.getLength();
-//            }
-//            var l2 = shape.getLengthTo(cutDist2);
-//            if (l2<RS.PointTolerance && cutDist2.isEnd===true) {
-//                l2 = shape.getLength();
-//            }
+            // make sure point 1 is closer to the start of the polyline:
+            if (cutDist1 > cutDist2) {
+                var dummy = cutDist1;
+                cutDist1 = cutDist2;
+                cutDist2 = dummy;
+            }
 
-            // TODO: use real click point (position)
-//            if (l1 > l2) {
-//                rest1.trimEndPoint(cutDist2, cutDist2);
-//                segment.trimStartPoint(cutDist2, cutDist2);
-//                segment.trimEndPoint(cutDist1, cutDist1);
-//                rest2.trimStartPoint(cutDist1, cutDist1);
-//            }
-//            else {
-                rest1.trimEndPoint(cutDist1);
+            rest1.trimEndPoint(cutDist1);
 
-                var l1 = segment.getLength();
-                segment.trimStartPoint(cutDist1);
-                var l2 = segment.getLength();
-                segment.trimEndPoint(cutDist2 - (l1-l2));
+            var l1 = segment.getLength();
+            segment.trimStartPoint(cutDist1);
+            var l2 = segment.getLength();
+            segment.trimEndPoint(cutDist2 - (l1-l2));
 
-                rest2.trimStartPoint(cutDist2);
-//            }
+            rest2.trimStartPoint(cutDist2);
         }
 
         if (!isNull(segment)) {
@@ -1428,7 +1430,7 @@ ShapeAlgorithms.createCircleFrom3Points = function(point1, point2, point3) {
 
     var ret = RCircle.createFrom3Points(point1, point2, point3);
 
-    if (isNull(ret)) {
+    if (!ret.isValid()) {
         ShapeAlgorithms.error = qsTr("No circle possible.");
         return undefined;
     }
@@ -1495,9 +1497,9 @@ ShapeAlgorithms.getCompleteQuadrilateralSegments = function(line1, line2, line3,
     var ret = [];
 
     // maps vertices to number of intersection points at that vertex:
-    var vertices = new Map(function(v1, v2) { return v1.equalsFuzzy(v2); });
+    var vertices = new MapCompat(function(v1, v2) { return v1.equalsFuzzy(v2); });
 
-    var lines = ShapeAlgorithms.removeSharedPointer([ line1, line2, line3, line4 ]);
+    var lines = [ line1, line2, line3, line4 ];
     var i, k;
 
     for (i=0; i<lines.length; i++) {
@@ -1508,7 +1510,7 @@ ShapeAlgorithms.getCompleteQuadrilateralSegments = function(line1, line2, line3,
                 continue;
             }
 
-            ips = ips.concat(lines[i].getIntersectionPoints(lines[k], false));
+            ips = ips.concat(lines[i].getIntersectionPoints(getPtr(lines[k]), false));
         }
 
         if (ips===0) {
@@ -1734,6 +1736,10 @@ ShapeAlgorithms.splineToLineOrArc = function(spline, tolerance, linesOnly) {
  * Converts the given circle into an arc with the given start angle or 0.
  */
 ShapeAlgorithms.circleToArc = function(circle, startAngle) {
+    if (isNull(startAngle)) {
+        startAngle = 0.0;
+    }
+
     return circle.toArc(startAngle);
 };
 
@@ -1761,35 +1767,35 @@ ShapeAlgorithms.lineOrArcToPolyline = function(shape, numSegments) {
     return ret;
 };
 
-ShapeAlgorithms.removeSharedPointer = function(shape) {
-    if (isArray(shape)) {
-        var ret = [];
-        for (var i=0; i<shape.length; i++) {
-            ret.push(ShapeAlgorithms.removeSharedPointer(shape[i]));
-        }
-        return ret;
-    }
+//ShapeAlgorithms.removeSharedPointer = function(shape) {
+//    if (isArray(shape)) {
+//        var ret = [];
+//        for (var i=0; i<shape.length; i++) {
+//            ret.push(ShapeAlgorithms.removeSharedPointer(shape[i]));
+//        }
+//        return ret;
+//    }
 
-    if (isFunction(shape.data)) {
-        return shape.data().clone();
-    }
-    else {
-        return shape;
-    }
-};
+//    if (isFunction(shape.data)) {
+//        return shape.data().clone();
+//    }
+//    else {
+//        return shape;
+//    }
+//};
 
 /**
  * \return Array of shapes of type point, line, arc, circle, ellipse.
  */
 ShapeAlgorithms.explodeToTrimmable = function(shape) {
     if (isSplineShape(shape) && !RSpline.hasProxy()) {
-        return ShapeAlgorithms.removeSharedPointer(shape.getExploded());
+        return shape.getExploded();
     }
     if (isPolylineShape(shape)) {
-        return ShapeAlgorithms.removeSharedPointer(shape.getExploded());
+        return shape.getExploded();
     }
     if (isTriangleShape(shape)) {
-        return ShapeAlgorithms.removeSharedPointer(shape.getExploded());
+        return shape.getExploded();
     }
     return [ shape ];
 };
@@ -1816,11 +1822,28 @@ ShapeAlgorithms.ellipseToArcCircleEllipse = function(ellipse) {
  * Transforms the given arc, circle or ellipse into an ellipse
  * using the given function which projects an RVector to another RVector.
  *
- * \returns An REllipse or in special cases an RArc or RCircle.
+ * \returns An REllipse or in special cases an RArc or RCircle. A polyline if no ellipse proxy is available to generate inscribed ellipses.
  *
- * OBSOLETE use RShape::transformArc instead
+ * \todo use RShape::transformArc instead
  */
 ShapeAlgorithms.transformArc = function(arc, fun) {
+    var arcsAsPolylines = (RSettings.getStringValue("IsoProject/GroupArcHandling", "ArcsAsEllipses")==="ArcsAsPolylines");
+
+    if (!REllipse.hasProxy() || arcsAsPolylines) {
+        if (!isEllipseShape(arc)) {
+            var pl = arc.approximateWithLines(arc.getRadius()*2*Math.PI / 64, arc.getStartAngle());
+
+            var plProj = new RPolyline();
+            for (var i=0; i<pl.countVertices(); i++) {
+                var v = pl.getVertexAt(i);
+                fun(v);
+                plProj.appendVertex(v);
+            }
+
+            return plProj;
+        }
+    }
+
     var r1, r2;
 
     if (isEllipseShape(arc)) {
@@ -1832,22 +1855,23 @@ ShapeAlgorithms.transformArc = function(arc, fun) {
         r2 = new RVector(0, arc.getRadius());
     }
 
-    // get vertices at extremities:
+    // get vertices at extremities (quad around full circle, ordered):
     var v1 = arc.getCenter().operator_add(r1).operator_add(r2);
     var v2 = arc.getCenter().operator_add(r1).operator_subtract(r2);
     var v3 = arc.getCenter().operator_subtract(r1).operator_subtract(r2);
     var v4 = arc.getCenter().operator_subtract(r1).operator_add(r2);
 
-    // project those vertices:
+    // project those vertices (project quad):
     fun(v1);
     fun(v2);
     fun(v3);
     fun(v4);
 
-    //var ret = [];
-    // inscribe ellipse into vertices:
     var ellipse = ShapeAlgorithms.createEllipseInscribedFromVertices(v1, v2, v3, v4);
-    //ret.push(ellipse.copy());
+
+    if (isNull(ellipse)) {
+        return arc;
+    }
 
     if (isArcShape(arc) || isEllipseShape(arc)) {
         var sp = arc.getStartPoint();
@@ -1906,7 +1930,7 @@ ShapeAlgorithms.appendShapeToPolylineAuto = function(pl, shape) {
     if (pl.countVertices()>0 && pl.getEndPoint().equalsFuzzy(shape.getEndPoint())) {
         shape.reverse();
     }
-    pl.appendShape(shape);
+    pl.appendShape(getPtr(shape));
 };
 
 /**
